#ifndef Functions_1_h
#define Functions_1_h

#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>


/*______________________________CREATE_DATA_______________________________*/

void createData_with_stream() {
    std::string filename = "test_file.txt";
    std::ofstream file_out(filename);
    for (int i = 0; i < 100; ++i){
        file_out << std::setw(3) << " " <<(rand() % 101 - 50) << std::endl;
    }
}

template<typename Container>
void createData_with_generate() {
    std::string filename = "test_file.txt";
    std::ofstream file_out(filename);
    using Value_type = typename Container::value_type;
    Container cont(100);
    generate(cont.begin(), cont.end(),[] () {return (rand() % 101 - 50);});
    std::ostream_iterator<Value_type> file_i(file_out, " ");
    std::copy(cont.begin(), cont.end(), file_i);
}

/*______________________________LOAD_DATA_______________________________*/

template <typename Container>
Container loadData_with_stream() {
    std::string filename = "test_file.txt";
    std::ifstream file_in(filename);
    if (!file_in) {
        std::cerr << "Error opening file! " << std::endl;
        exit(EXIT_FAILURE);
    }
    Container cont;
    using Value_type = typename Container::value_type;
    Value_type num;
    while (!file_in.eof()) {
        file_in >> num;
        cont.push_back(num);
    }
    return cont;
}

template <typename Container>
Container loadData_with_copy() {
    std::string filename = "test_file.txt";
    std::ifstream file_in(filename);
    if (!file_in) {
        std::cerr << "Error opening file! " << std::endl;
        exit(EXIT_FAILURE);
    }
    using value_type = typename Container::value_type;
    Container cont(100);
    std::copy(std::istream_iterator<value_type>(file_in),
              std::istream_iterator<value_type>(), std::back_inserter(cont));
    return cont;
}

/*_______________________________MODIFY_VAR14_____________________________*/

template<typename Container>
void Modify_template(Container& modify_cont) {
    using Value_type = typename Container::value_type;
    Value_type sum = 0;
    Value_type negative_sum = 0;
    for (const auto& num : modify_cont) {
        if (num < 0) {
            negative_sum += num;
        }
    }
    for (auto& num : modify_cont) {
        num += negative_sum/2;
        sum += num;
    }
    modify_cont.push_back(sum);
    modify_cont.push_back(abs(sum / 100));
}

template<typename Container>
void Modify_with_iter(const Container& it_begin, const Container& it_end) {
    using Value_type = typename Container::value_type;
    Value_type negative_sum = 0;
    for (auto num = it_begin; num != it_end; ++num) {
            if (*num < 0) {
                negative_sum += *num;
            }
    }
    for (auto num = it_begin; num != it_end; ++num) {
        *num += negative_sum/2;
    }
}

template<typename Container>
void Modify_with_for_each(T_ld& modify_cont) {
    using Value_type = typename Container::value_type;
    Value_type negative_sum = 0;
    for_each(modify_cont.begin(), modify_cont.end(), [&negative_sum](const auto &num) {
        if (num < 0) {
            negative_sum += num;
        }
    });
    for_each(modify_cont.begin(), modify_cont.end(), [negative_sum](auto &num) {
        num += negative_sum/2;
    });
}

void Modify_with_transform_and_fstream() {
    std::string filename = "test_file.txt";
    std::ifstream file_in(filename);
    if (!file_in) {
        std::cerr << "Error opening file! " << std::endl;
        exit(EXIT_FAILURE);
    }
    using Value_type = typename T_ld::value_type;
    Value_type negative_sum = 0;
    std::string new_filename = "new_test_file.txt";
    std::ofstream file_out(new_filename);
    for_each(std::istream_iterator<Value_type>(file_in), std::istream_iterator<Value_type>(), [&negative_sum](const auto &num) {
        if (num < 0) {
           negative_sum += num;
        }
    });
    file_in.close();
    file_in.open(filename);
    std::transform(std::istream_iterator<Value_type>(file_in),
                   std::istream_iterator<Value_type>(),
                   std::ostream_iterator<Value_type>(file_out, " "),
                   [&negative_sum](auto num){return num += negative_sum/2;});
}

/*_______________________________OUTPUT_RESULT______________________________*/

template<typename Container>
void outputResult_with_fstream(const Container& cont) {
    std::string output_filename = "output_test_file.txt";
    std::ofstream file_out(output_filename);
    for (const auto &num : cont) {
        file_out << num << " ";
    }
}

template<typename Container>
void outputResult_with_copy(const Container& cont) {
    std::string output_filename = "output_test_file.txt";
    std::ofstream file_out(output_filename);
    using Value_type = typename Container::value_type;
    std::copy(cont.begin(), cont.end(), std::ostream_iterator<Value_type>(file_out, " "));
}

#endif
