#include "File.h"
#include <fstream>
#include <iomanip>
#include <algorithm>


int File::idx = 1;

File::File(const std::string& p_filename)
    : filename(p_filename)
{}
 

void File::add_to_file(){
    std::ofstream file(filename, std::ios::binary | std::ios::app);
    if (!file.is_open()) {
        std::cerr << "Error: file cannot be opened!" << std::endl;
        exit(EXIT_FAILURE);
    }
    bool end = 0;
    Automobile car;
    std::cout << "Enter the information of automobiles" << std::endl;
    while (!end) {
        std::cout << "Enter surname of owner, car make code, car make, requied mark of fuel, engine power, tank volume, fuel remaining, oil volume, fuel price, oil price" << std::endl;
        std::cin >> car;
        file.write(reinterpret_cast<const char*>(&car), sizeof(Automobile));
        std::cout << "Do you want to continue typing(0) or complete(1) ?" << std::endl;
        std::cin >> end;
    }
}


void File::make_map(std::map<int, Automobile>& p_map) {
    Automobile car;
    std::ifstream file(filename, std::ios::binary);
    while (file.read(reinterpret_cast<char*>(&car), sizeof(Automobile))) {
        p_map.insert(std::make_pair(File::idx, car));
        ++File::idx;
    }
}


void File::delete_from_file(int p_idx){
    std::map<int, Automobile> map;
    make_map(map);
    map.erase(p_idx);
    std::ofstream file(filename, std::ios::binary);
    if (!file.is_open()) {
        std::cerr << "Error: File cannot be opened" << std::endl;
        exit(EXIT_FAILURE);
    }
    for (const auto& car : map) {
        file.write(reinterpret_cast<const char*>(&car.second), sizeof(Automobile));
    }
}


void File::replace_in_file(int p_idx, const Automobile& p_car) {
    std::map<int, Automobile> map;
    make_map(map);
    std::ofstream file(filename, std::ios::binary);
    if (!file.is_open()) {
        std::cerr << "Error: File cannot be opened" << std::endl;
        exit(EXIT_FAILURE);
    }
    map[p_idx] = p_car;
    for (const auto& car : map) {
        file.write(reinterpret_cast<const char*>(&car.second), sizeof(Automobile));
    }
}


void File::convert_into_a_text() {
    std::map<int, Automobile> map;
    make_map(map);
    std::string new_filename = filename + " formated.txt";
    std::ofstream file(new_filename);
    if (!file.is_open()) {
        std::cerr << "Error: File cannot be opened" << std::endl;
        exit(EXIT_FAILURE);
    }
    
    file << std::setw(159) << std::setfill('-') << "-" << std::endl;
    file << std::setfill(' ');
    file << std::setw(90) << " Automobiles " << std::endl;
    file << std::setw(159) << std::setfill('-') << "-" << std::endl;
    file << std::setfill(' ');
    file << "| ID | Surname of owner | Car make code | Car make | Required mark of fuel | Engine power | Tank volume | Fuel remainig | Oil volume | Fuel price | Oil price |" << std::endl;
    file << std::setw(159) << std::setfill('-') << "-" << std::endl;
    file << std::setw(1) << std::setfill(' ');

    for (const auto& car : map) {
        file << "|" << std::setw(3) << car.first << " |"
             << std::setw(17) << car.second.surname_of_owner << " |"
             << std::setw(14) << car.second.car_make_code << " |"
             << std::setw(9) << car.second.car_make << " |"
             << std::setw(22) << car.second.required_mark_of_fuel << " |"
             << std::setw(13) << car.second.engine_power << " |"
             << std::setw(12) << car.second.tank_volume << " |"
             << std::setw(14) << car.second.fuel_remainig << " |"
             << std::setw(11) << car.second.oil_volume << " |"
             << std::setw(11) << car.second.fuel_price << " |"
             << std::setw(10) << car.second.oil_price << " |"
             << std::endl;
    }
}


void File::search_by_car_make(const std::string& p_car_make) {
    std::map<int, Automobile> map;
    make_map(map);
    std::vector<Automobile> search;
    for (const auto& car : map) {
        if (car.second.car_make == p_car_make) {
            search.push_back(car.second);
        }
    }
    std::ofstream fout("result_of_searching_by_car_make.bin", std::ios::binary);
    if (!fout.is_open()) {
        std::cerr << "Error: File cannot be opened" << std::endl;
        exit(EXIT_FAILURE);
    }
    for (const auto& car : search) {
        fout.write(reinterpret_cast<const char*>(&car), sizeof(Automobile));
    }
    std::cout << "The result of searching was written to result_of_searching_by_car_make.bin" << std::endl;
}


void File::search_by_fuel_mark(const std::string& p_required_mark_of_fuel) {
    std::map<int, Automobile> map;
    make_map(map);
    std::vector<Automobile> search;
    for (const auto& car : map) {
        if (car.second.required_mark_of_fuel == p_required_mark_of_fuel) {
            search.push_back(car.second);
        }
    }
    std::ofstream file("result_of_searching_by_fuel_mark.bin", std::ios::binary);
    if (!file.is_open()) {
        std::cerr << "Error: File cannot be opened" << std::endl;
        exit(EXIT_FAILURE);
    }
    for (const auto& car : search) {
        file.write(reinterpret_cast<const char*>(&car), sizeof(Automobile));
    }
    std::cout << "The result of searching was written to result_of_searching_by_fuel_mark.bin" << std::endl;
}


void File::search_by_engine_power(int p_engine_power) {
    std::map<int, Automobile> map;
    make_map(map);
    std::vector<Automobile> search;
    for (const auto& car : map) {
        if (car.second.engine_power == p_engine_power) { search.push_back(car.second);
        }
    }
    std::ofstream file("result_of_searching_by_engine_power.bin", std::ios::binary);
    if (!file.is_open()) {
        std::cerr << "Error: File cannot be opened" << std::endl;
        exit(EXIT_FAILURE);
    }
    for (const auto& car : search) {
        file.write(reinterpret_cast<const char*>(&car), sizeof(Automobile));
    }
    std::cout << "The result of searching was written to result_of_searching_by_engine_power.bin" << std::endl;
}
